# Generated by Django 5.1.1 on 2024-10-11 19:55

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djpay", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="bill",
            name="verified",
            field=models.BooleanField(default=False),
        ),
    ]
